@@ -3,6 +3,10 @@
 from urllib import request, error
 import json
 import time
+import datetime
+import socket
+
+TIMEOUT = 500
 
 def send_to_server(access_token, messages, name, server, version, log,
         insecure=False):
@@ -23,7 +27,7 @@ def send_to_server(access_token, messages, name, server, version, log,
         log.info('Sending data to %s', address)
         req = request.Request(address)
         req.add_header("Content-Type", "application/json")
-        response = request.urlopen(req, serialized)
+        response = request.urlopen(req, serialized, TIMEOUT)
         return json.loads(response.read().decode('utf-8'))
     except error.HTTPError as ex:
         log.warning('Error while sending to server: %s', str(ex))
@@ -38,22 +42,41 @@ def send_to_server(access_token, messages, name, server, version, log,
         except Exception as e:
             log.warning('Could not connect to %s', server)
 
-def dump_to_server(access_token, msg_queue, name, server, insecure, staging_queue,
-        version, log):
-    while not msg_queue.empty():
-        message = msg_queue.get()
-        staging_queue.put(message)
+def dump_to_server(access_token, msg_list, name, server, insecure, version, log, send_all=False):
+    #TODO(soumya) Change after we get data on ok_messages
+    # This request is temporary- it'll be removed in the next day or two.
+    send_all = False
+    try:
+        prefix = "http" if insecure else "https"
+        address = prefix + "://" + server + "/api/v1/nothing"
+        address += "?access_token={0}&ok_messages={1}".format(access_token,
+                len(msg_list))
+        req = request.Request(address)
+        response = request.urlopen(req, b"", 0.4)
+    except Exception as e:
+        pass
+
+    stop_time = datetime.datetime.now() + datetime.timedelta(milliseconds=TIMEOUT)
+    initial_length = len(msg_list)
+    while msg_list:
+        if not send_all and datetime.datetime.now() > stop_time:
+            break
+        message = msg_list[-1]
         try:
-            if not send_to_server(access_token, message, name, server, version, log, insecure):
-                staging_queue.get() #throw away successful message
-            else:
-                msg_queue.put(staging_queue.get())
+            response = send_to_server(access_token, message, name, server, version, log, insecure)
+            if response:
+                msg_list.pop()
+            if send_all:
+                print("Submitting project... {0}% complete".format(len(msg_list)*100/initial_length))
         except SoftwareUpdated:
+            print("ok was updated. We will now terminate this run of ok.")
             log.info('ok was updated. Abort now; messages will be sent '
                      'to server on next invocation')
             return
         except error.URLError as ex:
             log.warning('URLError: %s', str(ex))
+        except socket.timeout as ex:
+            log.warning("socket.timeout: %s", str(ex))
     return
 
 def server_timer():
